"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = flowComment;
exports.removeFlowComment = removeFlowComment;
exports.addFlowComment = addFlowComment;
exports.isFlowFile = isFlowFile;

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/* ::
type Comment = {
  [key: string]: any,
  value: string,
  ignore: boolean,
}

type Comments = Array<Comment>
*/
var FLOW_DIRECTIVE = '@flow';

function flowComment(programPath
/* : Object */
) {
  var file = programPath.get('file');
  removeFlowComment(file.hub.file.ast.comments);
  programPath.addComment('leading', " ".concat(FLOW_DIRECTIVE), true);
}

function removeFlowComment(comments
/* : Comments */
) {
  if (!Array.isArray(comments)) {
    throw new TypeError("Expected a Array<Comment>, got ".concat(_typeof(comments)));
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = comments[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var comment = _step.value;

      if (comment.value.includes(FLOW_DIRECTIVE)) {
        comment.value = comment.value.replace(FLOW_DIRECTIVE, '');

        if (!comment.value.replace(/\*/g, '').trim()) {
          comment.ignore = true;
        }
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
} // line ? /*  */ or // @flow


function addFlowComment(path
/* : Object */
) {
  if (!isFlowFile(path)) {
    path.addComment('leading', " ".concat(FLOW_DIRECTIVE), true);
  }
}

function isFlowFile(path
/* : Object */
) {
  var file = path.get('file');

  if (!file || !file.hub) {
    return false;
  }

  var comments = file.hub.file.ast.comments;

  if (comments) {
    return comments.some(function (v) {
      return v.value.includes(FLOW_DIRECTIVE);
    });
  }

  return false;
}